@@ -1,5 +1,6 @@
 // [[Rcpp::plugins(cpp11)]]
 #include <Rcpp.h>
+// #include <Rcpp/hash/IndexHash.h>
 using namespace Rcpp ;
 
 // TODO: Perhaps redo everything with data pointers and 2d group indices (instead of filling the 2d structure every time !): http://www.cplusplus.com/reference/vector/vector/data/
@@ -7,30 +8,32 @@ using namespace Rcpp ;
 
 // improve logical method ?
 
-template <int RTYPE>
-inline bool isnaNUM(typename Rcpp::traits::storage_type<RTYPE>::type x) {
-  return x != x;
-}
-
-template <int RTYPE>
-inline bool isnaOTH(typename Rcpp::traits::storage_type<RTYPE>::type x) {
-  return x == Vector<RTYPE>::get_na();
-}
-
 template <int RTYPE>
 IntegerVector fNdistinctImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
                               const SEXP& gs, bool narm) {
   int l = x.size();
   typedef typename Rcpp::traits::storage_type<RTYPE>::type storage_t;
-  auto isnanT = (RTYPE == REALSXP) ? isnaNUM<RTYPE> : isnaOTH<RTYPE>;
+  auto isnanT = (RTYPE == REALSXP) ? [](storage_t x) { return x != x; } :
+    [](storage_t x) { return x == Vector<RTYPE>::get_na(); };
+
+  unsigned int addr;
 
   if(ng == 0) {
     sugar::IndexHash<RTYPE> hash(x);
      if(narm) {
+      // unsigned int addr;
+      // int *data = hash.data;
+      // int &m = hash.m;
+      // int &size_ = hash.size_;
+      // auto get_addr = hash.get_addr; // sugar::IndexHash<RTYPE>::get_addr;
+      // auto not_equal = hash.not_equal; // sugar::IndexHash<RTYPE>::not_equal;
+      // IndexHash *hashptr = new hash;
+      // auto get_addr2 = (hash.*get_addr);
       for(int i = 0; i != l; ++i) {
-        if(isnanT(x[i])) continue;
-        unsigned int addr = hash.get_addr(x[i]);
-        while(hash.data[addr] && hash.not_equal(x[hash.data[addr] - 1], x[i])) {
+        storage_t val = hash.src[i];
+        if(isnanT(val)) continue;
+        addr = hash.get_addr(val);
+        while(hash.data[addr] && hash.not_equal(hash.src[hash.data[addr] - 1], val)) {
           ++addr;
           if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
         }
@@ -44,14 +47,16 @@ IntegerVector fNdistinctImpl(const Vector<RTYPE>& x, int ng, const IntegerVector
     }
     return IntegerVector::create(hash.size_);
   } else {
+    // unsigned int addr;
     if(l != g.size()) stop("length(g) must match length(x)");
-    std::vector<std::vector<storage_t> > gmap(ng);
+    std::vector<std::vector<storage_t> > gmap(ng+1);
     IntegerVector out(ng);
+    int *outm1 = out.begin()-1;
     if(Rf_isNull(gs)) {
-      for(int i = 0; i != l; ++i) ++out[g[i]-1];
+      for(int i = 0; i != l; ++i) ++outm1[g[i]];
       for(int i = 0; i != ng; ++i) {
         if(out[i] == 0) stop("group size of 0 encountered");
-        gmap[i] = std::vector<storage_t> (out[i]);
+        gmap[i+1] = std::vector<storage_t> (out[i]);
         out[i] = 0;
       }
       // memset(out, 0, sizeof(int)*ng); // Stable ? -> Nope, gives error
@@ -60,18 +65,19 @@ IntegerVector fNdistinctImpl(const Vector<RTYPE>& x, int ng, const IntegerVector
       if(ng != gsv.size()) stop("ng must match length(gs)");
       for(int i = 0; i != ng; ++i) {
         if(gsv[i] == 0) stop("group size of 0 encountered");
-        gmap[i] = std::vector<storage_t> (gsv[i]);
+        gmap[i+1] = std::vector<storage_t> (gsv[i]);
       }
     }
-    for(int i = 0; i != l; ++i) gmap[g[i]-1][out[g[i]-1]++] = x[i];
+    for(int i = 0; i != l; ++i) gmap[g[i]][outm1[g[i]]++] = x[i];
     if(narm) {
       for(int gr = 0; gr != ng; ++gr) {
-        const std::vector<storage_t>& temp = gmap[gr]; // good ? // const Vector<RTYPE>& // wrap()
-        sugar::IndexHash<RTYPE> hash(wrap(temp));
+        // const std::vector<storage_t>& temp = gmap[gr+1]; // good ? // const Vector<RTYPE>& // wrap()
+        sugar::IndexHash<RTYPE> hash(wrap(gmap[gr+1])); // temp
         for(int i = hash.n; i--; ) {
-          if(isnanT(temp[i])) continue;
-          unsigned int addr = hash.get_addr(temp[i]);
-          while(hash.data[addr] && hash.not_equal(temp[hash.data[addr] - 1], temp[i])) {
+          storage_t val = hash.src[i];
+          if(isnanT(val)) continue;
+          addr = hash.get_addr(val);
+          while(hash.data[addr] && hash.not_equal(hash.src[hash.data[addr] - 1], val)) {
             ++addr;
             if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
           }
@@ -84,7 +90,7 @@ IntegerVector fNdistinctImpl(const Vector<RTYPE>& x, int ng, const IntegerVector
       }
     } else {
       for(int gr = 0; gr != ng; ++gr) {
-        sugar::IndexHash<RTYPE> hash(wrap(gmap[gr]));
+        sugar::IndexHash<RTYPE> hash(wrap(gmap[gr+1]));
         hash.fill();
         out[gr] = hash.size_;
       }
@@ -135,6 +141,7 @@ IntegerVector fNdistinctImpl(const Vector<LGLSXP>& x, int ng, const IntegerVecto
     }
     return IntegerVector::create(Nunique);
   } else {
+    if(l != g.size()) stop("length(g) must match length(x)");
     IntegerVector out(ng);
     if(narm) {
       LogicalVector which(ng);
@@ -238,25 +245,28 @@ SEXP fNdistinctlCpp(const List& x, int ng = 0, const IntegerVector& g = 0,
 
 
 
-
-
 template <int RTYPE>
 SEXP fNdistinctmImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g,
                       const SEXP& gs, bool narm, bool drop) {
   int l = x.nrow(), col = x.ncol();
   typedef typename Rcpp::traits::storage_type<RTYPE>::type storage_t;
-  auto isnanT = (RTYPE == REALSXP) ? isnaNUM<RTYPE> : isnaOTH<RTYPE>;
+  auto isnanT = (RTYPE == REALSXP) ? [](storage_t x) { return x != x; } :
+    [](storage_t x) { return x == Vector<RTYPE>::get_na(); };
+
+  unsigned int addr;
+
 
   if(ng == 0) {
     IntegerVector out = no_init_vector(col);
     if(narm) {
       for(int j = col; j--; ) {
-        ConstMatrixColumn<RTYPE> column = x(_ , j);
-        sugar::IndexHash<RTYPE> hash(wrap(column)); // why wrap needed ?
+        // ConstMatrixColumn<RTYPE> column = x(_ , j);
+        sugar::IndexHash<RTYPE> hash(wrap(x(_ , j))); // wrap(column) // why wrap needed ?
         for(int i = 0; i != l; ++i) {
-          if(isnanT(column[i])) continue;
-          unsigned int addr = hash.get_addr(column[i]);
-          while(hash.data[addr] && hash.not_equal(column[hash.data[addr] - 1], column[i])) {
+          storage_t val = hash.src[i];
+          if(isnanT(val)) continue;
+          addr = hash.get_addr(val);
+          while(hash.data[addr] && hash.not_equal(hash.src[hash.data[addr] - 1], val)) {
             ++addr;
             if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
           }
@@ -269,8 +279,8 @@ SEXP fNdistinctmImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g,
       }
     } else {
       for(int j = col; j--; ) {
-        ConstMatrixColumn<RTYPE> column = x(_ , j);
-        sugar::IndexHash<RTYPE> hash(wrap(column)); // why wrap needed ?
+        // ConstMatrixColumn<RTYPE> column = x(_ , j);
+        sugar::IndexHash<RTYPE> hash(wrap(x(_ , j))); // wrap(column) // why wrap needed ?
         hash.fill();
         out[j] = hash.size_;
       }
@@ -283,13 +293,14 @@ SEXP fNdistinctmImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g,
     return out;
   } else {
     if(l != g.size()) stop("length(g) must match length(x)");
-    std::vector<std::vector<storage_t> > gmap(ng);
+    int ngp = ng+1;
+    std::vector<std::vector<storage_t> > gmap(ngp);
     IntegerMatrix out = no_init_matrix(ng, col);
-    std::vector<int> n(ng);
+    std::vector<int> n(ngp);
     if(Rf_isNull(gs)) {
       // memset(n, 0, sizeof(int)*ng);
-      for(int i = 0; i != l; ++i) ++n[g[i]-1];
-      for(int i = 0; i != ng; ++i) {
+      for(int i = 0; i != l; ++i) ++n[g[i]];
+      for(int i = 1; i != ngp; ++i) {
         if(n[i] == 0) stop("group size of 0 encountered");
         gmap[i] = std::vector<storage_t> (n[i]);
       }
@@ -298,22 +309,23 @@ SEXP fNdistinctmImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g,
       if(ng != gsv.size()) stop("ng must match length(gs)");
       for(int i = 0; i != ng; ++i) {
         if(gsv[i] == 0) stop("group size of 0 encountered");
-        gmap[i] = std::vector<storage_t> (gsv[i]);
+        gmap[i+1] = std::vector<storage_t> (gsv[i]);
       }
     }
     if(narm) {
       for(int j = col; j--; ) {
         ConstMatrixColumn<RTYPE> column = x(_ , j);
         IntegerMatrix::Column outj = out(_, j);
-        n.assign(ng, 0); // memset(n, 0, sizeof(int)*ng);
-        for(int i = 0; i != l; ++i) gmap[g[i]-1][n[g[i]-1]++] = column[i]; // reading in all the values. Better way ?
+        n.assign(ngp, 0);
+        for(int i = 0; i != l; ++i) gmap[g[i]][n[g[i]]++] = column[i]; // reading in all the values. Better way ?
         for(int gr = 0; gr != ng; ++gr) {
-          const std::vector<storage_t>& temp = gmap[gr]; // good ? // const Vector<RTYPE>& // wrap()
-          sugar::IndexHash<RTYPE> hash(wrap(temp));
+          // const std::vector<storage_t>& temp = gmap[gr+1]; // good ? // const Vector<RTYPE>& // wrap()
+          sugar::IndexHash<RTYPE> hash(wrap(gmap[gr+1])); // wrap(temp)
           for(int i = hash.n; i--; ) {
-            if(isnanT(temp[i])) continue;
-            unsigned int addr = hash.get_addr(temp[i]);
-            while(hash.data[addr] && hash.not_equal(temp[hash.data[addr] - 1], temp[i])) {
+            storage_t val = hash.src[i];
+            if(isnanT(val)) continue;
+            addr = hash.get_addr(val);
+            while(hash.data[addr] && hash.not_equal(hash.src[hash.data[addr] - 1], val)) {
               ++addr;
               if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
             }
@@ -329,10 +341,10 @@ SEXP fNdistinctmImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g,
       for(int j = col; j--; ) {
         ConstMatrixColumn<RTYPE> column = x(_ , j);
         IntegerMatrix::Column outj = out(_, j);
-        n.assign(ng, 0); // memset(n, 0, sizeof(int)*ng);
-        for(int i = 0; i != l; ++i) gmap[g[i]-1][n[g[i]-1]++] = column[i]; // reading in all the values. Better way ?
+        n.assign(ngp, 0);
+        for(int i = 0; i != l; ++i) gmap[g[i]][n[g[i]]++] = column[i]; // reading in all the values. Better way ?
         for(int gr = 0; gr != ng; ++gr) {
-          sugar::IndexHash<RTYPE> hash(wrap(gmap[gr]));
+          sugar::IndexHash<RTYPE> hash(wrap(gmap[gr+1]));
           hash.fill();
           outj[gr] = hash.size_;
         }
@@ -390,6 +402,7 @@ SEXP fNdistinctmImpl(const Matrix<LGLSXP>& x, int ng, const IntegerVector& g, co
     }
     return out;
   } else {
+    if(l != g.size()) stop("length(g) must match length(x)");
     IntegerMatrix out(ng, col); //  = no_init_matrix
     if(narm) {
       for(int j = col; j--; ) {
@@ -460,3 +473,46 @@ SEXP fNdistinctmImpl(const Matrix<EXPRSXP>& x, int ng, const IntegerVector& g, c
 SEXP fNdistinctmCpp(SEXP x, int ng = 0, IntegerVector g = 0, SEXP gs = R_NilValue, bool narm = true, bool drop = true) {
   RCPP_RETURN_MATRIX(fNdistinctmImpl, x, ng, g, gs, narm, drop);
 }
+
+
+// This brings down code size from 180 kb to 168 kb, but about 25% slower than above..
+// template <int RTYPE>
+// SEXP fNdistinctmImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g, const SEXP& gs, bool narm, bool drop) {
+//   int col = x.ncol();
+//   Matrix<RTYPE> out = (ng == 0) ? no_init_matrix(1, col) : no_init_matrix(ng, col);
+//   for(int j = col; j--; ) out(_, j) = fNdistinctImpl<RTYPE>(x(_, j), ng, g, gs, narm);
+//   if(drop) {
+//     Rf_setAttrib(out, R_DimSymbol, R_NilValue);
+//     Rf_setAttrib(out, R_NamesSymbol, colnames(x));
+//   } else {
+//     colnames(out) = colnames(x);
+//   }
+//   return out;
+// }
+//
+//
+// template <>
+// SEXP fNdistinctmImpl(const Matrix<CPLXSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, bool narm, bool drop) {
+//   stop("Not supported SEXP type!");
+// }
+//
+// template <>
+// SEXP fNdistinctmImpl(const Matrix<VECSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, bool narm, bool drop) {
+//   stop("Not supported SEXP type!");
+// }
+//
+// template <>
+// SEXP fNdistinctmImpl(const Matrix<RAWSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, bool narm, bool drop) {
+//   stop("Not supported SEXP type!");
+// }
+//
+// template <>
+// SEXP fNdistinctmImpl(const Matrix<EXPRSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, bool narm, bool drop) {
+//   stop("Not supported SEXP type!");
+// }
+//
+//
+// // [[Rcpp::export]]
+// SEXP fNdistinctmCpp(SEXP x, int ng = 0, IntegerVector g = 0, SEXP gs = R_NilValue, bool narm = true, bool drop = true) {
+//   RCPP_RETURN_MATRIX(fNdistinctmImpl, x, ng, g, gs, narm, drop);
+// }
