@@ -28,7 +28,7 @@ o = order(o)
 od = order(rnorm(length(td)))
 muo = m[od, ]
 datauo = data[od, ]
-guo = as.factor.GRP(g)[od]
+guo = as.factor_GRP(g)[od]
 tduo = td[od]
 t2duo = seq_along(od)[od]
 od = order(od)
@@ -51,6 +51,10 @@ test_that("flag performs like baselag", {
   expect_equal(flag(m, 2, stubs = FALSE), dapply(m, baselag, 2))
   expect_equal(flag(mNA, stubs = FALSE), dapply(mNA, baselag))
   expect_equal(flag(mNA, 2, stubs = FALSE), dapply(mNA, baselag, 2))
+  expect_equal(flag(num_vars(data), stubs = FALSE), dapply(num_vars(data), baselag))
+  expect_equal(flag(num_vars(data), 2, stubs = FALSE), dapply(num_vars(data), baselag, 2))
+  expect_equal(flag(num_vars(dataNA), stubs = FALSE), dapply(num_vars(dataNA), baselag))
+  expect_equal(flag(num_vars(dataNA), 2, stubs = FALSE), dapply(num_vars(dataNA), baselag, 2))
   expect_equal(flag(x, 1, f), BY(x, f, baselag, use.g.names = FALSE))
   expect_equal(flag(x, 2, f), BY(x, f, baselag, 2, use.g.names = FALSE))
   expect_equal(flag(xNA, 1, f), BY(xNA, f, baselag, use.g.names = FALSE))
@@ -295,6 +299,10 @@ test_that("F performs like baselead", {
   expect_equal(F(m, -2, stubs = FALSE), dapply(m, baselead, 2))
   expect_equal(F(mNA, -1, stubs = FALSE), dapply(mNA, baselead))
   expect_equal(F(mNA, -2, stubs = FALSE), dapply(mNA, baselead, 2))
+  expect_equal(F(num_vars(data), -1, stubs = FALSE), dapply(num_vars(data), baselead))
+  expect_equal(F(num_vars(data), -2, stubs = FALSE), dapply(num_vars(data), baselead, 2))
+  expect_equal(F(num_vars(dataNA), -1, stubs = FALSE), dapply(num_vars(dataNA), baselead))
+  expect_equal(F(num_vars(dataNA), -2, stubs = FALSE), dapply(num_vars(dataNA), baselead, 2))
   expect_equal(F(x, -1, f), BY(x, f, baselead, use.g.names = FALSE))
   expect_equal(F(x, -2, f), BY(x, f, baselead, 2, use.g.names = FALSE))
   expect_equal(F(xNA, -1, f), BY(xNA, f, baselead, use.g.names = FALSE))
