@@ -11,6 +11,7 @@ wNA[sample.int(100,20)] <- NA
 wdatNA[sample.int(32, 5)] <- NA
 f <- as.factor(sample.int(10, 100, TRUE))
 g <- GRP(mtcars, ~ cyl + vs + am)
+gf <- as.factor_GRP(g)
 mtcNA <- na_insert(mtcars)
 mtcNA[27,1] <- NA # single group NA !!
 m <- as.matrix(mtcars)
@@ -18,6 +19,22 @@ mNA <- as.matrix(mtcNA)
 mNAc <- mNA
 storage.mode(mNAc) <- "character"
 
+wmean <- function(x, w, na.rm = FALSE) {
+  if(na.rm) {
+    cc <- complete.cases(x, w)
+    x <- x[cc]
+    w <- w[cc]
+  }
+  sum(x*w)/sum(w)
+}
+
+wBY <- function(x, f, FUN, w, ...) {
+  if(is.atomic(x) && !is.array(x)) return(mapply(FUN, split(x, f), split(w, f), ...))
+  wspl <- split(w, f)
+  if(is.atomic(x)) return(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...)))
+  qDF(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...), return = "matrix"))
+}
+
 
 test_that("fmean performs like base::mean", {
   expect_equal(fmean(NA), mean(NA))
@@ -37,6 +54,10 @@ test_that("fmean performs like base::mean", {
   expect_equal(fmean(m, na.rm = FALSE), dapply(m, mean))
   expect_equal(fmean(mNA, na.rm = FALSE), dapply(mNA, mean))
   expect_equal(fmean(mNA), dapply(mNA, mean, na.rm = TRUE))
+  expect_equal(fmean(mtcars), dapply(mtcars, mean, na.rm = TRUE))
+  expect_equal(fmean(mtcars, na.rm = FALSE), dapply(mtcars, mean))
+  expect_equal(fmean(mtcNA, na.rm = FALSE), dapply(mtcNA, mean))
+  expect_equal(fmean(mtcNA), dapply(mtcNA, mean, na.rm = TRUE))
   expect_equal(fmean(x, f), BY(x, f, mean, na.rm = TRUE))
   expect_equal(fmean(x, f, na.rm = FALSE), BY(x, f, mean))
   expect_equal(fmean(xNA, f, na.rm = FALSE), BY(xNA, f, mean))
@@ -120,51 +141,75 @@ test_that("fmean performs like fmean with weights all equal", {
   expect_equal(fmean(mtcNA, g), fmean(mtcNA, g, rep(999.9999,32)))
 })
 
-test_that("fmean with weights performs like stats::weighted.mean", {
+test_that("fmean with weights performs like wmean (defined above)", {
   # complete weights
-  expect_equal(fmean(NA, w = 1), weighted.mean(NA, 1))
-  expect_equal(fmean(NA, w = 1, na.rm = FALSE), weighted.mean(NA, 1))
-  expect_equal(fmean(1, w = 1), weighted.mean(1, w = 1))
-  expect_equal(fmean(1:3, w = 1:3), weighted.mean(1:3, 1:3))
-  expect_equal(fmean(-1:1, w = 1:3), weighted.mean(-1:1, 1:3))
-  expect_equal(fmean(1, w = 1, na.rm = FALSE), weighted.mean(1, 1))
-  expect_equal(fmean(1:3, w = c(0.99,3454,1.111), na.rm = FALSE), weighted.mean(1:3, c(0.99,3454,1.111)))
-  expect_equal(fmean(-1:1, w = 1:3, na.rm = FALSE), weighted.mean(-1:1, 1:3))
-  expect_equal(fmean(x, w = w), weighted.mean(x, w))
-  expect_equal(fmean(x, w = w, na.rm = FALSE), weighted.mean(x, w))
-  expect_equal(fmean(xNA, w = w, na.rm = FALSE), weighted.mean(xNA, w))
-  expect_equal(fmean(xNA, w = w), weighted.mean(xNA, w, na.rm = TRUE))
+  expect_equal(fmean(NA, w = 1), wmean(NA, 1))
+  expect_equal(fmean(NA, w = 1, na.rm = FALSE), wmean(NA, 1))
+  expect_equal(fmean(1, w = 1), wmean(1, w = 1))
+  expect_equal(fmean(1:3, w = 1:3), wmean(1:3, 1:3))
+  expect_equal(fmean(-1:1, w = 1:3), wmean(-1:1, 1:3))
+  expect_equal(fmean(1, w = 1, na.rm = FALSE), wmean(1, 1))
+  expect_equal(fmean(1:3, w = c(0.99,3454,1.111), na.rm = FALSE), wmean(1:3, c(0.99,3454,1.111)))
+  expect_equal(fmean(-1:1, w = 1:3, na.rm = FALSE), wmean(-1:1, 1:3))
+  expect_equal(fmean(x, w = w), wmean(x, w))
+  expect_equal(fmean(x, w = w, na.rm = FALSE), wmean(x, w))
+  expect_equal(fmean(xNA, w = w, na.rm = FALSE), wmean(xNA, w))
+  expect_equal(fmean(xNA, w = w), wmean(xNA, w, na.rm = TRUE))
   expect_equal(fmean(mtcars, w = wdat), fmean(m, w = wdat))
-  expect_equal(fmean(m, w = wdat), dapply(m, weighted.mean, wdat, na.rm = TRUE))
-  expect_equal(fmean(m, w = wdat, na.rm = FALSE), dapply(m, weighted.mean, wdat))
-  expect_equal(fmean(mNA, w = wdat, na.rm = FALSE), dapply(mNA, weighted.mean, wdat))
-  expect_equal(fmean(mNA, w = wdat), dapply(mNA, weighted.mean, wdat, na.rm = TRUE))
-  expect_equal(fmean(x, f, w), mapply(weighted.mean, split(x, f), split(w, f)))
-  expect_equal(fmean(x, f, w, na.rm = FALSE), mapply(weighted.mean, split(x, f), split(w, f)))
-  expect_equal(fmean(xNA, f, w, na.rm = FALSE), mapply(weighted.mean, split(xNA, f), split(w, f)))
-  expect_equal(fmean(xNA, f, w), mapply(weighted.mean, split(xNA, f), split(w, f), na.rm = TRUE))
+  expect_equal(fmean(m, w = wdat), dapply(m, wmean, wdat, na.rm = TRUE))
+  expect_equal(fmean(m, w = wdat, na.rm = FALSE), dapply(m, wmean, wdat))
+  expect_equal(fmean(mNA, w = wdat, na.rm = FALSE), dapply(mNA, wmean, wdat))
+  expect_equal(fmean(mNA, w = wdat), dapply(mNA, wmean, wdat, na.rm = TRUE))
+  expect_equal(fmean(mtcars, w = wdat), dapply(mtcars, wmean, wdat, na.rm = TRUE))
+  expect_equal(fmean(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wmean, wdat))
+  expect_equal(fmean(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wmean, wdat))
+  expect_equal(fmean(mtcNA, w = wdat), dapply(mtcNA, wmean, wdat, na.rm = TRUE))
+  expect_equal(fmean(x, f, w), wBY(x, f, wmean, w))
+  expect_equal(fmean(x, f, w, na.rm = FALSE), wBY(x, f, wmean, w))
+  expect_equal(fmean(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wmean, w))
+  expect_equal(fmean(xNA, f, w), wBY(xNA, f, wmean, w, na.rm = TRUE))
+  expect_equal(fmean(m, g, wdat), wBY(m, gf, wmean, wdat))
+  expect_equal(fmean(m, g, wdat, na.rm = FALSE), wBY(m, gf, wmean, wdat))
+  expect_equal(fmean(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, gf, wmean, wdat))
+  expect_equal(fmean(mNA, g, wdat), wBY(mNA, gf, wmean, wdat, na.rm = TRUE))
+  expect_equal(fmean(mtcars, g, wdat), wBY(mtcars, gf, wmean, wdat))
+  expect_equal(fmean(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, gf, wmean, wdat))
+  expect_equal(fmean(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, gf, wmean, wdat))
+  expect_equal(fmean(mtcNA, g, wdat), wBY(mtcNA, gf, wmean, wdat, na.rm = TRUE))
   # missing weights
-  expect_equal(fmean(NA, w = NA), weighted.mean(NA, NA))
-  expect_equal(fmean(NA, w = NA, na.rm = FALSE), weighted.mean(NA, NA))
-  expect_equal(fmean(1, w = NA), weighted.mean(1, w = NA))
-  # expect_equal(fmean(1:3, w = c(NA,1:2)), weighted.mean(1:3, c(NA,1:2), na.rm = TRUE)) # weighted mean doesn not remove NA's from weights !!
-  # expect_equal(fmean(-1:1, w = c(NA,1:2)), weighted.mean(-1:1, c(NA,1:2), na.rm = TRUE))
-  expect_equal(fmean(1, w = NA, na.rm = FALSE), weighted.mean(1, NA))
-  expect_equal(fmean(1:3, w = c(NA,1:2), na.rm = FALSE), weighted.mean(1:3, c(NA,1:2)))
-  expect_equal(fmean(-1:1, w = c(NA,1:2), na.rm = FALSE), weighted.mean(-1:1, c(NA,1:2)))
-  # expect_equal(fmean(x, w = wNA), weighted.mean(x, wNA, na.rm = TRUE))
-  expect_equal(fmean(x, w = wNA, na.rm = FALSE), weighted.mean(x, wNA))
-  expect_equal(fmean(xNA, w = wNA, na.rm = FALSE), weighted.mean(xNA, wNA))
-  # expect_equal(fmean(xNA, w = wNA), weighted.mean(xNA, wNA, na.rm = TRUE))
+  expect_equal(fmean(NA, w = NA), wmean(NA, NA))
+  expect_equal(fmean(NA, w = NA, na.rm = FALSE), wmean(NA, NA))
+  expect_equal(fmean(1, w = NA), wmean(1, w = NA))
+  expect_equal(fmean(1:3, w = c(NA,1:2)), wmean(1:3, c(NA,1:2), na.rm = TRUE))
+  expect_equal(fmean(-1:1, w = c(NA,1:2)), wmean(-1:1, c(NA,1:2), na.rm = TRUE))
+  expect_equal(fmean(1, w = NA, na.rm = FALSE), wmean(1, NA))
+  expect_equal(fmean(1:3, w = c(NA,1:2), na.rm = FALSE), wmean(1:3, c(NA,1:2)))
+  expect_equal(fmean(-1:1, w = c(NA,1:2), na.rm = FALSE), wmean(-1:1, c(NA,1:2)))
+  expect_equal(fmean(x, w = wNA), wmean(x, wNA, na.rm = TRUE))
+  expect_equal(fmean(x, w = wNA, na.rm = FALSE), wmean(x, wNA))
+  expect_equal(fmean(xNA, w = wNA, na.rm = FALSE), wmean(xNA, wNA))
+  expect_equal(fmean(xNA, w = wNA), wmean(xNA, wNA, na.rm = TRUE))
   expect_equal(fmean(mtcars, w = wdatNA), fmean(m, w = wdatNA))
-  # expect_equal(fmean(m, w = wdatNA), dapply(m, weighted.mean, wdatNA, na.rm = TRUE))
-  expect_equal(fmean(m, w = wdatNA, na.rm = FALSE), dapply(m, weighted.mean, wdatNA))
-  expect_equal(fmean(mNA, w = wdatNA, na.rm = FALSE), dapply(mNA, weighted.mean, wdatNA))
-  # expect_equal(fmean(mNA, w = wdatNA), dapply(mNA, weighted.mean, wdatNA, na.rm = TRUE))
-  # expect_equal(fmean(x, f, wNA), mapply(weighted.mean, split(x, f), split(wNA, f), na.rm = TRUE))
-  expect_equal(fmean(x, f, wNA, na.rm = FALSE), mapply(weighted.mean, split(x, f), split(wNA, f)))
-  expect_equal(fmean(xNA, f, wNA, na.rm = FALSE), mapply(weighted.mean, split(xNA, f), split(wNA, f)))
-  # expect_equal(fmean(xNA, f, wNA), mapply(weighted.mean, split(xNA, f), split(wNA, f), na.rm = TRUE))
+  expect_equal(fmean(m, w = wdatNA), dapply(m, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(m, w = wdatNA, na.rm = FALSE), dapply(m, wmean, wdatNA))
+  expect_equal(fmean(mNA, w = wdatNA, na.rm = FALSE), dapply(mNA, wmean, wdatNA))
+  expect_equal(fmean(mNA, w = wdatNA), dapply(mNA, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(mtcars, w = wdatNA), dapply(mtcars, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wmean, wdatNA))
+  expect_equal(fmean(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wmean, wdatNA))
+  expect_equal(fmean(mtcNA, w = wdatNA), dapply(mtcNA, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(x, f, wNA), wBY(x, f, wmean, wNA, na.rm = TRUE))
+  expect_equal(fmean(x, f, wNA, na.rm = FALSE), wBY(x, f, wmean, wNA))
+  expect_equal(fmean(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wmean, wNA))
+  expect_equal(fmean(xNA, f, wNA), wBY(xNA, f, wmean, wNA, na.rm = TRUE))
+  expect_equal(fmean(m, g, wdatNA), wBY(m, gf, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(m, g, wdatNA, na.rm = FALSE), wBY(m, gf, wmean, wdatNA))
+  expect_equal(fmean(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, gf, wmean, wdatNA))
+  expect_equal(fmean(mNA, g, wdatNA), wBY(mNA, gf, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(mtcars, g, wdatNA), wBY(mtcars, gf, wmean, wdatNA, na.rm = TRUE))
+  expect_equal(fmean(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, gf, wmean, wdatNA))
+  expect_equal(fmean(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, gf, wmean, wdatNA))
+  expect_equal(fmean(mtcNA, g, wdatNA), wBY(mtcNA, gf, wmean, wdatNA, na.rm = TRUE))
 })
 
 test_that("fmean performs numerically stable", {
@@ -321,6 +366,7 @@ test_that("fmean produces errors for wrong input", {
   expect_error(fmean(1:2,1:3))
   expect_error(fmean(m,1:31))
   expect_error(fmean(mtcars,1:31))
+  expect_error(fmean(mtcars, w = 1:31))
   expect_error(fmean("a", w = 1))
   expect_error(fmean(1:2, w = 1:3))
   expect_error(fmean(NA_character_, w = 1))
