@@ -43,6 +43,10 @@ test_that("fmin performs like base::min", {
   expect_equal(fmin(m, na.rm = FALSE), dapply(m, min))
   expect_equal(fmin(mNA, na.rm = FALSE), dapply(mNA, min))
   expect_equal(fmin(mNA), dapply(mNA, min, na.rm = TRUE))
+  expect_equal(fmin(mtcars), dapply(mtcars, min, na.rm = TRUE))
+  expect_equal(fmin(mtcars, na.rm = FALSE), dapply(mtcars, min))
+  expect_equal(fmin(mtcNA, na.rm = FALSE), dapply(mtcNA, min))
+  expect_equal(fmin(mtcNA), dapply(mtcNA, min, na.rm = TRUE))
   expect_equal(fmin(x, f), BY(x, f, min, na.rm = TRUE))
   expect_equal(fmin(x, f, na.rm = FALSE), BY(x, f, min))
   expect_equal(fmin(xNA, f, na.rm = FALSE), BY(xNA, f, min))
@@ -136,6 +140,10 @@ test_that("fmax performs like base::max", {
   expect_equal(fmax(m, na.rm = FALSE), dapply(m, max))
   expect_equal(fmax(mNA, na.rm = FALSE), dapply(mNA, max))
   expect_equal(fmax(mNA), dapply(mNA, max, na.rm = TRUE))
+  expect_equal(fmax(mtcars), dapply(mtcars, max, na.rm = TRUE))
+  expect_equal(fmax(mtcars, na.rm = FALSE), dapply(mtcars, max))
+  expect_equal(fmax(mtcNA, na.rm = FALSE), dapply(mtcNA, max))
+  expect_equal(fmax(mtcNA), dapply(mtcNA, max, na.rm = TRUE))
   expect_equal(fmax(x, f), BY(x, f, max, na.rm = TRUE))
   expect_equal(fmax(x, f, na.rm = FALSE), BY(x, f, max))
   expect_equal(fmax(xNA, f, na.rm = FALSE), BY(xNA, f, max))
